@@ -25,10 +25,10 @@ import pathlib
 import time
 import unittest
 import uuid
+from typing import Optional
 
 import psutil
 import pytest
-import pkg_resources
 
 from google.cloud.bigquery._pandas_helpers import _BIGNUMERIC_SUPPORT
 from . import helpers
@@ -63,6 +63,7 @@ from google.cloud import bigquery
 from google.cloud import bigquery_v2
 from google.cloud.bigquery.dataset import Dataset
 from google.cloud.bigquery.dataset import DatasetReference
+from google.cloud.bigquery.schema import SchemaField
 from google.cloud.bigquery.table import Table
 from google.cloud._helpers import UTC
 from google.cloud.bigquery import dbapi, enums
@@ -89,6 +90,12 @@ SCHEMA = [
     bigquery.SchemaField("full_name", "STRING", mode="REQUIRED"),
     bigquery.SchemaField("age", "INTEGER", mode="REQUIRED"),
 ]
+CLUSTERING_SCHEMA = [
+    bigquery.SchemaField("full_name", "STRING", mode="REQUIRED"),
+    bigquery.SchemaField("age", "INTEGER", mode="REQUIRED"),
+    bigquery.SchemaField("body_height_cm", "INTEGER", mode="REQUIRED"),
+    bigquery.SchemaField("date_of_birth", "DATE", mode="REQUIRED"),
+]
 TIME_PARTITIONING_CLUSTERING_FIELDS_SCHEMA = [
     bigquery.SchemaField("transaction_time", "TIMESTAMP", mode="REQUIRED"),
     bigquery.SchemaField("transaction_id", "INTEGER", mode="REQUIRED"),
@@ -116,13 +123,6 @@ retry_storage_errors = RetryErrors(
     (TooManyRequests, InternalServerError, ServiceUnavailable)
 )
 
-PYARROW_MINIMUM_VERSION = pkg_resources.parse_version("0.17.0")
-
-if pyarrow:
-    PYARROW_INSTALLED_VERSION = pkg_resources.get_distribution("pyarrow").parsed_version
-else:
-    PYARROW_INSTALLED_VERSION = None
-
 MTLS_TESTING = os.getenv("GOOGLE_API_USE_CLIENT_CERTIFICATE") == "true"
 
 
@@ -131,7 +131,7 @@ def _has_rows(result):
 
 
 def _make_dataset_id(prefix):
-    return "%s%s" % (prefix, unique_resource_id())
+    return f"python_bigquery_tests_system_{prefix}{unique_resource_id()}"
 
 
 def _load_json_schema(filename="schema.json"):
@@ -150,7 +150,7 @@ class Config(object):
     global state.
     """
 
-    CLIENT = None
+    CLIENT: Optional[bigquery.Client] = None
     CURSOR = None
     DATASET = None
 
@@ -197,7 +197,9 @@ class TestBigQuery(unittest.TestCase):
     def _create_bucket(self, bucket_name, location=None):
         storage_client = storage.Client()
         bucket = storage_client.bucket(bucket_name)
-        retry_storage_errors(bucket.create)(location=location)
+        retry_storage_errors(storage_client.create_bucket)(
+            bucket_name, location=location
+        )
         self.to_delete.append(bucket)
 
         return bucket
@@ -436,6 +438,22 @@ class TestBigQuery(unittest.TestCase):
         with self.assertRaises(exceptions.BadRequest):
             Config.CLIENT.delete_dataset(dataset)
 
+    def test_delete_job_metadata(self):
+        dataset_id = _make_dataset_id("us_east1")
+        self.temp_dataset(dataset_id, location="us-east1")
+        full_table_id = f"{Config.CLIENT.project}.{dataset_id}.test_delete_job_metadata"
+        table = Table(full_table_id, schema=[SchemaField("col", "STRING")])
+        Config.CLIENT.create_table(table)
+        query_job: bigquery.QueryJob = Config.CLIENT.query(
+            f"SELECT COUNT(*) FROM `{full_table_id}`", location="us-east1",
+        )
+        query_job.result()
+        self.assertIsNotNone(Config.CLIENT.get_job(query_job))
+
+        Config.CLIENT.delete_job_metadata(query_job)
+        with self.assertRaises(NotFound):
+            Config.CLIENT.get_job(query_job)
+
     def test_get_table_w_public_dataset(self):
         public = "bigquery-public-data"
         dataset_id = "samples"
@@ -567,6 +585,25 @@ class TestBigQuery(unittest.TestCase):
             self.assertEqual(found.field_type, expected.field_type)
             self.assertEqual(found.mode, expected.mode)
 
+    def test_update_table_clustering_configuration(self):
+        dataset = self.temp_dataset(_make_dataset_id("update_table"))
+
+        TABLE_NAME = "test_table"
+        table_arg = Table(dataset.table(TABLE_NAME), schema=CLUSTERING_SCHEMA)
+        self.assertFalse(_table_exists(table_arg))
+
+        table = helpers.retry_403(Config.CLIENT.create_table)(table_arg)
+        self.to_delete.insert(0, table)
+        self.assertTrue(_table_exists(table))
+
+        table.clustering_fields = ["full_name", "date_of_birth"]
+        table2 = Config.CLIENT.update_table(table, ["clustering_fields"])
+        self.assertEqual(table2.clustering_fields, ["full_name", "date_of_birth"])
+
+        table2.clustering_fields = None
+        table3 = Config.CLIENT.update_table(table2, ["clustering_fields"])
+        self.assertIsNone(table3.clustering_fields, None)
+
     @staticmethod
     def _fetch_single_page(table, selected_fields=None):
         iterator = Config.CLIENT.list_rows(table, selected_fields=selected_fields)
@@ -880,7 +917,7 @@ class TestBigQuery(unittest.TestCase):
         job_id = load_job.job_id
 
         # Can get the job from the EU.
-        load_job = client.get_job(job_id, location="EU")
+        load_job = client.get_job(load_job)
         self.assertEqual(job_id, load_job.job_id)
         self.assertEqual("EU", load_job.location)
         self.assertTrue(load_job.exists())
@@ -897,7 +934,7 @@ class TestBigQuery(unittest.TestCase):
 
         # Can cancel the job from the EU.
         self.assertTrue(load_job.cancel())
-        load_job = client.cancel_job(job_id, location="EU")
+        load_job = client.cancel_job(load_job)
         self.assertEqual(job_id, load_job.job_id)
         self.assertEqual("EU", load_job.location)
 
@@ -1212,8 +1249,7 @@ class TestBigQuery(unittest.TestCase):
         # Even though the query takes >1 second, the call to getQueryResults
         # should succeed.
         self.assertFalse(query_job.done(timeout=1))
-
-        Config.CLIENT.cancel_job(query_job.job_id, location=query_job.location)
+        self.assertIsNotNone(Config.CLIENT.cancel_job(query_job))
 
     def test_query_w_page_size(self):
         page_size = 45
